<?php

namespace App\Factory;

use App\Exception;

class ProductFactory {

    private $connection;

    public function __construct (\Doctrine\DBAL\Connection $connection) {
        $this->connection = $connection;
    }
    
    public function getProduct ($product) {
        $sql = "
            SELECT CDPRODUTO, CDARVPROD, NMPRODUTO, IDTIPOCOMPPROD
              FROM PRODUTO PR
             WHERE CDARVPROD  = :CDARVPROD
                OR CDPRODINTE = :CDPRODINTE
                OR CDPRODUTO  = :CDPRODUTO
        ";

        $productCode = str_replace(".", "", $product);
        $productCode = preg_replace("/[^0-9A-Z]/", "", $productCode);

        $params = array(
            ":CDARVPROD"  => $productCode,
            ':CDPRODINTE' => $productCode,
            ':CDPRODUTO'  => $productCode
        );
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getProductByObsCode ($observationCode) {
        $obs  = explode("|", $observationCode);

        $obs0 = preg_match("/[0-9a-zA-Z]{2}/", $obs[0]);
    	$obs1 = preg_match("/[0-9a-zA-Z]{2}/", $obs[1]);
    	
    	if(strlen($obs[0]) != 2 || !$obs0)
    	    throw new Exception\ParamsException("
    	        O Código Grupo de Observação ({$obs[0]}) no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
    	if(strlen($obs[1]) != 2 || !$obs1)
    	    throw new Exception\ParamsException("
    	        O Código de Observação ({$obs[1]}), no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
        
        $sql = "
            SELECT P.CDPRODUTO, P.CDARVPROD, P.NMPRODUTO, P.IDTIPOCOMPPROD
              FROM OCORRENCIA O
              JOIN PRODUTO P
                ON O.CDPRODUTO = P.CDPRODUTO
             WHERE O.CDGRPOCOR = :CDGRPOCOR
               AND O.CDOCORR = :CDOCORR
        ";

        $params = array(
            ":CDGRPOCOR" => $obs[0],
            ":CDOCORR" => $obs[1]
        );
        
        return $this->connection->fetchAll($sql, $params);
    }
    
    public function getCombinedConfTela($CDFILIAL, $CDCAIXA) {
        $sqlCombinedConfTela = "
            SELECT NRCONFTELA 
    		  FROM CONFTELACAIXA
    		 WHERE CDFILICAIXA = :CDFILIAL
    		   AND CDCAIXA  = :CDCAIXA
    		   AND DTINIVIGENCIA = (SELECT DTINIVIGENCIA FROM (SELECT DTINIVIGENCIA 
    		    					  FROM CONFTELACAIXA 
                                     WHERE SYSDATE >= DTINIVIGENCIA 
    					               AND CDFILICAIXA = :CDFILIAL 
    					               AND CDCAIXA  = :CDCAIXA
                                  ORDER BY DTINIVIGENCIA DESC) WHERE ROWNUM = 1)";
                              
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':CDCAIXA'  => $CDCAIXA
        ];
        
        $result = $this->connection->fetchAll($sqlCombinedConfTela, $params);
        return $result[0]['NRCONFTELA'];
    }
    
    public function getCombinedProducts($CDFILIAL, $CDLOJA, $NRCONFTELA) {
        $sqlCombinedProds = "
            SELECT I.NRBUTTON, I.DSBUTTON, I.NRLIMPRODCOM, L.IDTIPCOBRA
              FROM ITMENUCONFTE I
              JOIN LOJA L ON L.CDFILIAL = I.CDFILIAL
             WHERE I.CDFILIAL = :CDFILIAL
               AND L.CDLOJA = :CDLOJA
               AND I.NRCONFTELA = :NRCONFTELA
               AND I.IDTPBUTTON = 'B'
          ORDER BY I.NRBUTTON
        ";
        
        $params = [
            ':CDFILIAL'   => $CDFILIAL,
            ':CDLOJA'     => $CDLOJA,
            ':NRCONFTELA' => $NRCONFTELA
        ];
        
        // Gets the combined product "buttons", which are the containers for the actual combined products.
        return $this->connection->fetchAll($sqlCombinedProds, $params);
    }
    
    public function getCombinedChilds($CDFILIAL, $NRCONFTELA) {
        $sqlCombinedChilds = "
            SELECT NVL(I.DSBUTTON, P.NMPRODUTO) AS NMPRODUTO, P.CDARVPROD, P.CDPRODUTO, P.IDTIPOPROD, P.IDTIPOCOMPPROD
              FROM PRODUTO P LEFT JOIN ITMENUCONFTE I
                                    ON P.CDPRODUTO = I.CDIDENTBUTON
                                   AND I.CDFILIAL = :CDFILIAL
                                   AND I.NRCONFTELA = :NRCONFTELA
            WHERE P.IDTIPOPROD IS NOT NULL
        ";
        
        $params = [
            ':CDFILIAL' => $CDFILIAL,
            ':NRCONFTELA'  => $NRCONFTELA
        ];
        
        // Gets all the actual combined products. These must be sorted into their respective buttons.
        return $this->connection->fetchAll($sqlCombinedChilds, $params);
    }
    
    public function insertProduct ($product, $consumerName) {
        $sql = "
            INSERT INTO ITCOMANDAVEN
    			(CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN,
    			CDPRODUTO, QTPRODCOMVEN, VRPRECCOMVEN, TXPRODCOMVEN,
    			IDSTPRCOMVEN, VRDESCCOMVEN, NRLUGARMESA, DTHRINCOMVEN,
    			IDPRODIMPFIS, CDLOJA, VRACRCOMVEN, NRSEQPRODCOM, NRSEQPRODCUP,
    			VRPRECCLCOMVEN, CDCAIXACOLETOR, CDPRODPROMOCAO, CDVENDEDOR, NRINSCRCONS, DSOBSPEDDIGCMD, CDSENHAPED, NMCONSVEND)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN,
    			:CDPRODUTO, :QTPRODCOMVEN, :VRPRECCOMVEN, :TXPRODCOMVEN,
    			:IDSTPRCOMVEN, :VRDESCCOMVEN, :NRLUGARMESA, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'),
    			:IDPRODIMPFIS, :CDLOJA, :VRACRCOMVEN, :NRSEQPRODCOM, :NRSEQPRODCUP,
    			:VRPRECCLCOMVEN, :CDCAIXACOLETOR, :CDPRODPROMOCAO, :CDVENDEDOR, :NRINSCRCONS, :DSOBSPEDDIGCMD, :CDSENHAPED, :NMCONSVEND)";
    		
	    date_default_timezone_set($this->getCurrentTimezone($product["SGESTADO"]));
	    
    	$params = array(
    	    ":CDFILIAL"       => $product["CDFILIAL"],
			":NRVENDAREST"    => $product["NRVENDAREST"],
			":NRCOMANDA"      => $product["NRCOMANDA"],
			":NRPRODCOMVEN"   => $product["NRPRODCOMVEN"],
			":CDPRODUTO"      => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODUTO"]),
			":QTPRODCOMVEN"   => $this->toFloat($product["QTPRODCOMVEN"]),
			":VRPRECCOMVEN"   => $this->toFloat($product["VRPRECCOMVEN"]),
			":TXPRODCOMVEN"   => preg_replace('/[^\x00-\x7F]+/', '', $product["TXPRODCOMVEN"]),
			":IDSTPRCOMVEN"   => $product["IDSTPRCOMVEN"],
			":VRDESCCOMVEN"   => $this->toFloat($product["VRDESCCOMVEN"]),
			":NRLUGARMESA"    => $product["NRLUGARMESA"],
			":CURRDATE"       => date("d/m/Y H:i:s"),
			":IDPRODIMPFIS"   => $product["IDPRODIMPFIS"],
			":CDLOJA"         => $product["CDLOJA"],
			":VRACRCOMVEN"    => $this->toFloat($product["VRACRCOMVEN"]),
			":NRSEQPRODCOM"   => $product["NRSEQPRODCOM"],
			":NRSEQPRODCUP"   => $product["NRSEQPRODCUP"],
			":VRPRECCLCOMVEN" => $this->toFloat($product["VRPRECCLCOMVEN"]),
			":CDCAIXACOLETOR" => $product["CDCAIXACOLETOR"],
			":CDPRODPROMOCAO" => preg_replace("/[^0-9A-Z]/", "", $product["CDPRODPROMOCAO"]),
			":CDVENDEDOR"     => $product["CDVENDEDOR"],
			":NRINSCRCONS"    => $product["NRINSCRCONS"],
			":DSOBSPEDDIGCMD" => preg_replace('/[^\x00-\x7F]+/', '', $product["TXPRODCOMVEN"]),
			":CDSENHAPED"     => $product["CDSENHAPED"],
			":NMCONSVEND"     => substr(strtoupper($this->removeAcentos($consumerName)), 0, 50)
    	);
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function getObservation ($observationCode) {

        $obs  = explode("|", $observationCode);

        $obs0 = preg_match("/[0-9a-zA-Z]{2}/", $obs[0]);
    	$obs1 = preg_match("/[0-9a-zA-Z]{2}/", $obs[1]);
    	
        if(strlen($obs[0]) != 2 || !$obs0)
    	    throw new Exception\ParamsException("
    	        O Código Grupo de Observação ({$obs[0]}) no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
    	if(strlen($obs[1]) != 2 || !$obs1)
    	    throw new Exception\ParamsException("
    	        O Código de Observação ({$obs[1]}), no código externo {$observationCode} deve possuir dois caracteres alfanuméricos, sem caracteres especiais.");
    	
        //===================== Busca por CDGRPOCOR =====================
    	
    	$sql = "
    		SELECT *
    		  FROM OCORRENCIA
    		 WHERE CDGRPOCOR = :CDGRPOCOR";

    	$params = array(
    		":CDGRPOCOR" => $obs[0]
    	);
    	
    	$cdgrpocor = $this->connection->fetchAssoc($sql, $params);
    	
    	if(!$cdgrpocor)
    	     throw new Exception\ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Shoppub.");

    	//===================== Busca por CDOCOR =====================
    	
    	$sql = "
    		SELECT *
    		   FROM OCORRENCIA
    		  WHERE CDGRPOCOR = :CDGRPOCOR
    		    AND CDOCORR = :CDOCORR";

    	$params = array(
    	    ":CDGRPOCOR" => $obs[0],
    		  ":CDOCORR" => $obs[1]
    	);
    	
    	$cdocor = $this->connection->fetchAssoc($sql, $params);
    	if(!$cdocor)
    	     throw new Exception\ParamsException("
    	        O código externo {$observationCode} não foi encontrado.<br>
    	        Verifique no Odhen Retail o código externo desta observação e faça a correção no portal Shoppub.");
        
        return $cdocor;
    }
    
    public function insertObservation ($obs) {
        $sql = "
            INSERT INTO	OBSITCOMANDAVEN
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRPRODCOMVEN, CDOCORR, CDGRPOCOR, NRPRODCOMVENOBS, QTPRODCOMVENOBS)
            VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRPRODCOMVEN, :CDOCORR, :CDGRPOCOR, :NRPRODCOMVENOBS, :QTPRODCOMVENOBS)";
		
	    $params = array(
	        ":CDFILIAL"        => $obs["CDFILIAL"],
            ":NRVENDAREST"     => $obs["NRVENDAREST"],
            ":NRCOMANDA"       => $obs["NRCOMANDA"],
            ":NRPRODCOMVEN"    => $obs["NRPRODCOMVEN"],
            ":CDOCORR"         => $obs["CDOCORR"],
            ":CDGRPOCOR"       => $obs["CDGRPOCOR"],
            ":NRPRODCOMVENOBS" => $obs["NRPRODCOMVENOBS"],
            ":QTPRODCOMVENOBS" => $obs["QTPRODCOMVENOBS"]  
        );
        
        $this->connection->executeQuery($sql, $params);
    }
    
    private function tofloat($num) {
        $dotPos = strrpos($num, '.');
        $commaPos = strrpos($num, ',');
        $sep = (($dotPos > $commaPos) && $dotPos) ? $dotPos : 
            ((($commaPos > $dotPos) && $commaPos) ? $commaPos : false);
       
        if (!$sep) {
            return floatval(preg_replace("/[^0-9]/", "", $num));
        } 
    
        $tmp = floatval(
            preg_replace("/[^0-9]/", "", substr($num, 0, $sep)) . '.' .
            preg_replace("/[^0-9]/", "", substr($num, $sep+1, strlen($num)))
        );
	    
	    return  str_replace(".", ",", $tmp);
    }
    
    /*
     * Função que busca dentro do json de TimeZones do projeto
     * o estado do cliente para buscar o Default Time Zone correto para o php
     * 
     */
    private function getCurrentTimezone ($state) {
        $timezones = array(
            "AC" => "America/Rio_branco",   "AL" => "America/Maceio",
            "AP" => "America/Belem",        "AM" => "America/Manaus",
            "BA" => "America/Bahia",        "CE" => "America/Fortaleza",
            "DF" => "America/Fortaleza",    "ES" => "America/Fortaleza",
            "GO" => "America/Fortaleza",    "MA" => "America/Fortaleza",
            "MT" => "America/Cuiaba",       "MS" => "America/Campo_Grande",
            "MG" => "America/Fortaleza",    "PR" => "America/Fortaleza",
            "PB" => "America/Fortaleza",    "PA" => "America/Belem",
            "PE" => "America/Recife",       "PI" => "America/Fortaleza",
            "RJ" => "America/Fortaleza",    "RN" => "America/Fortaleza",
            "RS" => "America/Fortaleza",    "RO" => "America/Porto_Velho",
            "RR" => "America/Boa_Vista",    "SC" => "America/Fortaleza",
            "SE" => "America/Maceio",       "SP" => "America/Fortaleza",
            "TO" => "America/Araguaia",    
        );
       
        return $timezones[$state];
    }
    
    private function removeAcentos($texto) {
        return preg_replace(array("/(á|à|ã|â|ä)/","/(Á|À|Ã|Â|Ä)/","/(é|è|ê|ë)/","/(É|È|Ê|Ë)/","/(í|ì|î|ï)/","/(Í|Ì|Î|Ï)/","/(ó|ò|õ|ô|ö)/","/(Ó|Ò|Õ|Ô|Ö)/","/(ú|ù|û|ü)/","/(Ú|Ù|Û|Ü)/","/(ñ)/","/(Ñ)/","/(ç)/","/(Ç)/"),explode(" ","a A e E i I o O u U n N c C"),$texto);
    }
    
}
